from django.contrib import admin

from swap_user.admin import BaseUserAdmin
from swap_user.to_phone_otp.forms import AddPhoneUserForm, EditPhoneUserForm
from swap_user.to_phone_otp.models import PhoneOTPUser


class PhoneOTPUserAdmin(BaseUserAdmin):
    """
    Admin class with overridden `add` and `edit` forms.
    """

    add_form_class = AddPhoneUserForm
    change_form_class = EditPhoneUserForm


admin.site.register(PhoneOTPUser, PhoneOTPUserAdmin)



from django.contrib.auth.base_user import AbstractBaseUser as DjangoAbstractBaseUser
from django.contrib.auth.models import PermissionsMixin
from django.db import models
from django.utils.translation import gettext_lazy as _

from swap_user.to_email.managers import EmailUserManager


class AbstractEmailUser(PermissionsMixin, DjangoAbstractBaseUser):
    """
    Abstract EmailUser implementation - subclass this class to provide your own
    custom User class with `email` field.

    Provides fields:
    - email
    - is_active (required by django.contrib.admin)
    - is_staff (required by django.contrib.admin)

    Provides attributes:
    - USERNAME_FIELD
    - EMAIL_FIELD

    REQUIRED_FIELDS by default will include USERNAME_FIELD and password.
    """

    email = models.EmailField(verbose_name=_("email address"), unique=True,)
    is_active = models.BooleanField(
        verbose_name=_("active"),
        default=False,
        help_text=_(
            "Designates whether this user should be treated as active. "
            "Unselect this instead of deleting accounts."
        ),
    )
    is_staff = models.BooleanField(
        verbose_name=_("staff status"),
        default=False,
        help_text=_("Designates whether the user can log into this admin site."),
    )

    objects = EmailUserManager()

    USERNAME_FIELD = "email"
    EMAIL_FIELD = "email"

    class Meta:
        abstract = True

    def __str__(self):
        return self.email

    clean = DjangoAbstractBaseUser.clean
    get_short_name = __str__
    get_full_name = __str__



from django.conf import settings as django_settings
from django.utils.module_loading import import_string


#
# Mostly inspired by `rest_framework.settings`
#

IMPORT_SETTINGS = [
    "OTP_SENDER_CLASS",
    "GET_OTP_SERVICE_CLASS",
    "CHECK_OTP_SERVICE_CLASS",
    "VALIDATION_SERVICE_CLASS",
]
DEFAULT_SETTINGS = {
    # services
    "OTP_SENDER_CLASS": "swap_user.otp.senders.StdOutOTPSender",
    "GET_OTP_SERVICE_CLASS": "swap_user.otp.services.GetOTPService",
    "CHECK_OTP_SERVICE_CLASS": "swap_user.otp.services.CheckOTPService",
    "VALIDATION_SERVICE_CLASS": "swap_user.otp.services.ValidationService",
    # patterns
    "INVALID_LOGIN_COUNTER_PATTERN": "swap-user:invalid:login:{user_id}",
    "SENT_OTP_COUNTER_PATTERN": "swap-user:sent:otp:{user_id}",
    "USER_BANNED_FOR_INVALID_LOGIN": "swap-user:banned:invalid:login:{user_id}",
    "USER_BANNED_FOR_OTP_RATE_LIMIT_PATTERN": "swap-user:banned:rate:limit:{user_id}",
    "OTP_PATTERN": "swap-user:otp:{user_id}",
    # constant values
    "BAN_FOR_INVALID_LOGIN_TIMEOUT": 60 * 60 * 24,  # 24 hours
    "BAN_FOR_OTP_RATE_LIMIT_TIMEOUT": 60 * 60 * 2,  # 2 hours
    "OTP_TIMEOUT": 60,  # 60 seconds
    "MAX_ATTEMPTS_OF_INVALID_LOGIN": 3,
    "MAX_NUMBER_OF_OTP_SENT": 5,
    "OTP_ALPHABET": "0123456789",
    "OTP_LENGTH": 5,
}
NAMESPACE = "SWAP_USER"

# Add default values and create working mapping
MAPPING = {
    "A": "PHONENUMBER_DB_FORMAT",
    "B": "PHONENUMBER_DEFAULT_REGION",
    "C": "PHONENUMBER_DEFAULT_FORMAT",
}


class SwapUserSettings:
    """
    Settings object mostly inspired from `rest_framework.settings`.
    """

    def make_import(self, item, path):
        val = import_string(path)
        setattr(self, item, val)

        return val

    def __getattr__(self, item):
        try:
            namespaced_settings = getattr(django_settings, NAMESPACE)
            value = namespaced_settings[item]
        except (AttributeError, KeyError):
            value = DEFAULT_SETTINGS[item]

        if item in IMPORT_SETTINGS:
            value = self.make_import(item, value)

        return value


swap_user_settings = SwapUserSettings()



from swap_user.to_phone_otp.base_managers import BasePhoneOTPUserManager


class PhoneOTPUserManager(BasePhoneOTPUserManager):
    """
    Concrete implementation of manager for PhoneOTPUser.
    """

    pass



from swap_user.to_phone.base_models import AbstractPhoneUser


class PhoneUser(AbstractPhoneUser):
    """
    Concrete implementation of user with phone login.
    """

    class Meta:
        swappable = "AUTH_USER_MODEL"
        verbose_name = "phone user"
        verbose_name_plural = "phone users"



from django import forms
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _

from swap_user.helpers import normalize_username
from swap_user.settings import swap_user_settings


UserModel = get_user_model()


class GetOTPForm(forms.ModelForm):
    """
    Unfortunately we can't check User exist or not at this screen by the
    security reasons - if we will show error when User doesn't exist,
    attacker can just check all the emails / phones.
    """

    class Meta:
        model = UserModel
        fields = (UserModel.USERNAME_FIELD,)

    def clean(self):
        """
        We are preventing unique validation by overriding this method and
        adding extra check for user ban.
        """

        username_field = UserModel.USERNAME_FIELD
        raw_username = self.cleaned_data[username_field]
        username = normalize_username(raw_username)

        service_class = swap_user_settings.VALIDATION_SERVICE_CLASS
        service = service_class()

        service.check_user_is_banned_for_otp_rate_limit(username)
        service.check_user_is_banned_for_invalid_login_attempts(username)
        service.check_extra(username=username)

        return self.cleaned_data


class CheckOTPForm(GetOTPForm):
    """
    Here we are checking User presence among with OPT check.
    """

    otp = forms.CharField(label=_("OTP"), widget=forms.PasswordInput(attrs={"autofocus": True}))

    def clean(self):
        """
        Method that allows to us pass through few validation checks:
            - User DB presence
            - OTP password validity
        """

        username_field = UserModel.USERNAME_FIELD
        username = self.cleaned_data[username_field]
        otp = self.cleaned_data["otp"]

        service_class = swap_user_settings.VALIDATION_SERVICE_CLASS
        service = service_class()

        service.check_user_is_banned_for_otp_rate_limit(username)
        service.check_user_is_banned_for_invalid_login_attempts(username)
        service.check_password(username, otp)
        service.check_extra(username=username, otp=otp)

        return self.cleaned_data



from django.apps import AppConfig


class PhoneOTPUserConfig(AppConfig):
    # `name` for INSTALLED_APPS
    name = "swap_user.to_phone_otp"
    # `label` used for relations reference in format `app_label.Model`
    # also it used for `AUTH_USER_MODEL` settings and fixtures definition
    label = "swap_to_phone_otp"
    # `verbose name` for Admin panel display
    verbose_name = "Django Swap User"
    # we are maintaining historical behavior
    default_auto_field = "django.db.models.AutoField"
